package com.ruoyi.web.controller.common;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.atomic.AtomicInteger;

@Component
@ServerEndpoint(value = "/appVerify/ws/{sessionId}")
public class WebSocket {

    private static Logger log = LoggerFactory.getLogger(WebSocket.class);

    /**
     * 在线连接数
     */
    private static final AtomicInteger OnlineCount = new AtomicInteger(0);
    /**
     * 用线程安全的CopyOnWriteArraySet来存放客户端连接的信息
     */
    private static CopyOnWriteArraySet<Client> socketServers = new CopyOnWriteArraySet<>();

    /**
     * websocket封装的session,信息推送，就是通过它来信息推送
     */
    private Session session;
    /**
     * 服务端的userName,因为用的是set，每个客户端的username必须不一样，否则会被覆盖。
     * 要想完成ui界面聊天的功能，服务端也需要作为客户端来接收后台推送用户发送的信息
     */
    private final static String SYS_USERNAME = "admin";

    @PostConstruct
    public void init() {
        log.info("websocket====>init方法被执行");
    }
    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam(value = "sessionId") String sessionId) {
        this.session = session;
        socketServers.add(new Client(sessionId, session));
        int cnt = OnlineCount.incrementAndGet(); // 在线数加1
        log.info("有连接加入，当前连接数为：{},sessionId为: {}", cnt,sessionId);
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session) {

        socketServers.forEach(client -> {
            if (client.getSession().getId().equals(session.getId())) {
                int cnt = OnlineCount.decrementAndGet();
                log.info("客户端:[{}]断开连接，当前连接数为：{}", client.getSessionId(),cnt);
                socketServers.remove(client);
            }
        });
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message
     *            客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        log.info("来自客户端的消息：{}",message);
        SendMessage(session, "服务器端接收到消息："+message);

    }

    /**
     * 出现错误
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("发生错误：{}，Session ID： {}",error.getMessage(),session.getId());
        error.printStackTrace();
    }

    /**
     * 发送消息，实践表明，每次浏览器刷新，session会发生变化。
     * @param session
     * @param message
     */
    public static void SendMessage(Session session, String message) {
        try {
//            session.getBasicRemote().sendText(String.format("%s (From Server，Session ID=%s)",message,session.getId()));
            session.getBasicRemote().sendText(message);
        } catch (IOException e) {
            log.error("发送消息出错：{}", e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * 群发消息
     * @param message
     * @throws IOException
     */
    public static void BroadCastInfo(String message) throws IOException {
        //群发，不能发送给服务端自己
//        socketServers.stream().filter(cli -> cli.getsessionId() != SYS_USERNAME)
//                .forEach(client -> {
//                    try {
//                        client.getSession().getBasicRemote().sendText(message);
//                    } catch (IOException e) {
//                        e.printStackTrace();
//                    }
//                });
//        log.info("服务端推送给所有客户端 :[{}]", message);
    }

    /**
     * 根据sessionId发消息,sessionId为前端发送的唯一标识,用户扫码时需要
     * @param sessionId 会话id
     * @param message
     */
    public synchronized static void SendMessage(String message,String sessionId) {
        socketServers.forEach(client -> {
            if (sessionId.equals(client.getSessionId())) {
                try {
                    synchronized(client.getSession()){
                        client.getSession().getBasicRemote().sendText(message);
                        log.info("服务端推送给客户端消息[{}] : [{}]", client.getSessionId(), message);
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }else {
                log.warn("没有找到你指定ID的会话：{}",sessionId);
            }
        });

    }
}
