package com.ruoyi.web.controller.iot.receive;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.common.utils.map.GpsCoordinateUtils;
import com.ruoyi.common.utils.onenet.LogEntry;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.domain.EtAdminOrder;
import com.ruoyi.system.domain.EtOnlineLog;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.service.*;
import com.ruoyi.web.controller.iot.domain.BodyObj;
import com.ruoyi.web.controller.iot.util.Util;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;
import org.locationtech.jts.geom.Geometry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

/**
 * 接收硬件参数
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/common")
public class ReceiveController {

    private static final Logger log = LoggerFactory.getLogger(ReceiveController.class);

    @Value(value = "${iot.token}")
    private String token;

    @Resource
    private IAsDeviceService asDeviceService;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtAdminOrderService etAdminOrderService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private IEtOnlineLogService etOnlineLogService;

    private final Object lock = new Object();

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysConfigService sysConfigService;

    // 用于记录上次发送命令的时间
    private static AtomicLong lastCommandTime = new AtomicLong(0);
    private static final long COMMAND_COOLDOWN_MS = 5 * 60 * 1000; // 5分钟

    private static final Map<String, Long> commandCooldownMap = new ConcurrentHashMap<>();
    private static final long COOLDOWN_PERIOD_MS = 10000; // 冷却时间为 10 秒


    /**
     * 功能描述：第三方平台数据接收。<p>
     *           <ul>注:
     *               <li>1.OneNet平台为了保证数据不丢失，有重发机制，如果重复数据对业务有影响，数据接收端需要对重复数据进行排除重复处理。</li>
     *               <li>2.OneNet每一次post数据请求后，等待客户端的响应都设有时限，在规定时限内没有收到响应会认为发送失败。
     *                    接收程序接收到数据时，尽量先缓存起来，再做业务逻辑处理。</li>
     *           </ul>
     * @param body 数据消息
     * @return 任意字符串。OneNet平台接收到http 200的响应，才会认为数据推送成功，否则会重发。
     */
    @RequestMapping(value = "/receive",method = RequestMethod.POST)
    @ResponseBody
    @SneakyThrows
    @Transactional
    public String receive(@RequestBody String body){

        log.info("receive方法接收到参数:  body String --- " +body);
        /************************************************
         *  解析数据推送请求，非加密模式。
         *  如果是明文模式使用以下代码 hangdleBody(body, false);
         **************************************************/
        /*************明文模式  start****************/
        BodyObj obj = Util.resolveBody(body, false);
        log.info("receive方法解析对象:  body Object --- " + JSON.toJSONString(obj));
        log.info("接收到receive方法时间:  " + System.currentTimeMillis());
        // 使用线程池提交任务
        scheduledExecutorService.submit(() -> {
            synchronized (lock) {
                try {
                    handleBody(obj);
                } catch (UnsupportedEncodingException | NoSuchAlgorithmException | InvalidKeyException e) {
                    e.printStackTrace();
                }
            }
        });
        /*************明文模式  end****************/
        return "ok";
    }

    private void handleBody(BodyObj obj) throws UnsupportedEncodingException, NoSuchAlgorithmException, InvalidKeyException {
        if (obj != null){
            boolean dataRight = Util.checkSignature(obj, token);

            if (dataRight){
                // 记录开始时间
                long startTime = System.nanoTime();

                log.info("receive方法验证签名正确: content" + JSON.toJSONString(obj));
                String msg = (String) obj.getMsg();
                log.info("receive方法-获取到消息体: msg---" +msg);
                JSONObject jsonObject = JSONObject.parseObject(msg, JSONObject.class);
                String devName = (String)jsonObject.get("dev_name");
                /*异步更新在线状态*/
                AsDevice asDevice = asDeviceMapper.selectAsDeviceByMac(devName);
                asynchronousUpdateOnlineStatus(asDevice);
                if(IotConstants.ONENET_LOCATION.equals(jsonObject.get("ds_id")) && ObjectUtil.isNotNull(jsonObject.get("value"))){
                    LogEntry logEntry = JSONObject.parseObject(msg, LogEntry.class);
                    log.info("logEntry转换后的对象: logEntry---【{}】" , JSON.toJSONString(logEntry));
                    LogEntry.LocationValue value = logEntry.getValue();
                    if(ObjectUtil.isNull(asDevice)){
                        throw new ServiceException("未找到车辆信息");
                    }
                    if(ObjectUtil.isNotNull(asDevice)){
                        // 将msg的定位信息保存到redis中
                        redisCache.setCacheObject(CacheConstants.CACHE_DEVICE_KEY+asDevice.getSn(),msg);
                        log.info("reids更新定位成功==========================>" +asDevice.getSn());

                        // 坐标转换  WGS84 转 GCJ02
                        double[] doubles = coordinateConvert(value);
                        BigDecimal lat = new BigDecimal(doubles[0]).setScale(8, RoundingMode.HALF_UP);
                        BigDecimal lon = new BigDecimal(doubles[1]).setScale(8, RoundingMode.HALF_UP);
                        log.info("转换后的GCJ02经纬度：" + lon + "---" + lat);

                        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(asDevice.getAreaId());
                        //  相差一分钟以上的消息不做处理
                        boolean oneMinuteDifference = DateUtils.oneMinuteDifference(logEntry.getAt(), System.currentTimeMillis());
                        if(ObjectUtil.isNotNull(area) && !oneMinuteDifference){
                            /** 2. 判断是否在禁行区内  如果在， 根据配置‘禁行区内断电配置’进行断电 **/
                            String isAdminUnlocking = asDevice.getIsAdminUnlocking();// 是否是管理员开锁：0-否；1-是
                            boolean noRidingArea = isNoRidingArea(value, asDevice, area, isAdminUnlocking);
                            /** 3.超出运营区外断电 包含靠近运营区播报 */
                            outAreaOutage(value, asDevice, lon, lat, area, isAdminUnlocking, noRidingArea);
                            /** 4.锁同步关锁 */
                            lockSynchronization(msg, asDevice, logEntry, value, asDevice, lon, lat);
                            /** 5.低电量 生成换电工单*/
                            replacementOrder(asDevice, area);
                        }
                    }else{
                        log.info("未找到车辆对象：" +logEntry.getDevName());
                    }
                }
                // 计算执行时间（以毫秒为单位）
                long duration = (System.nanoTime() - startTime) / 1_000_000;
                // 输出执行时间
                log.info("===============回调方法执行时间============= " + duration + " 毫秒");
            }else {
                log.info("receive方法验证签名错误: signature error");
            }
        }else {
            log.info("receive方法参数为空: body empty error");
        }
    }

//    移动到定时任务中处理
//    /**
//     * 异步保存定位
//     */
//    private void asynchronousSaveLog(String msg, long at,String mac,BigDecimal lon,BigDecimal lat,AsDevice device) {
//        //异步保存定位
//        scheduledExecutorService.schedule(() -> {
//            EtLocationLog etLocationLog = new EtLocationLog();
//            etLocationLog.setOnenetMsg(msg);
//            etLocationLog.setCreateTime(DateUtils.getNowDate());
//            etLocationLog.setLongitude(lon.toString());
//            etLocationLog.setLatitude(lat.toString());
//            etLocationLog.setMac(mac);
//            etLocationLog.setAt(new Date(at));
//            etLocationLog.setStatus(device.getStatus());
//            etLocationLog.setLockStatus(device.getLockStatus());
//            etLocationLogMapper.insertEtLocationLog(etLocationLog);
//        }, 0, TimeUnit.SECONDS);
//    }
//
//    /** 无定位更新设备 */
//    private void noLocationUpdateDevice(LogEntry logEntry, LogEntry.LocationValue value, AsDevice device) {
//        Integer bat = value.getBat();
//        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
//        log.info("保存电压：" + divide);
//        device.setVoltage(divide.toString());//电压
//        EtModel model = etModelService.selectEtModelByModelId(device.getModelId());
//        // 根据电压计算续航里程
//        if(ObjectUtil.isNotNull(model)){
//            Integer remainingMileage = 0;
//            if(StrUtil.isNotBlank(device.getVoltage())){
//                remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
//            }
//            Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
//            device.setRemainingMileage(remainingMileage);
//            device.setRemainingPower(electricQuantity.toString());
//        }
//        device.setLastTime(DateUtils.getNowDate());
//        device.setGps("0");
//        device.setSignalStrength(value.getCsq());
//        device.setSatellites(0);
//        device.setQuality(value.getQ());
//        int i = asDeviceService.updateLocation(device);
//        if(i>0){
//            log.info("未获取到定位===============保存电压等数值成功===========>" + logEntry.getDevName());
//        }
//    }
//
//    /** 计算电量和里程后更新设备*/
//    private int updateDevice(LogEntry.LocationValue value, AsDevice device, BigDecimal lon, BigDecimal lat) {
//        device.setLatitude(lat.toString());
//        device.setLongitude(lon.toString());
//        Integer bat = value.getBat();
//        BigDecimal divide = new BigDecimal(bat).divide(new BigDecimal(10));
//        log.info("保存电压：" + divide);
//        device.setVoltage(divide.toString());//电压
//        // 根据电压计算续航里程
//        EtModel model = etModelService.selectEtModelByModelId(device.getModelId());
//        if(ObjectUtil.isNotNull(model)){
//            Integer remainingMileage = 0;
//            if(StrUtil.isNotBlank(device.getVoltage())){
//                remainingMileage = CommonUtil.getRemainingMileage(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage(), model.getFullEndurance());
//            }
//            Integer electricQuantity = CommonUtil.getElectricQuantity(device.getVoltage(), model.getFullVoltage(), model.getLowVoltage());//电量百分百
//            device.setRemainingMileage(remainingMileage);
//            device.setRemainingPower(electricQuantity.toString());
//        }
//        device.setLastTime(DateUtils.getNowDate());
//        device.setLastLocationTime(DateUtils.getNowDate());
//        device.setGps("1");
//        // 信号强度
//        device.setSignalStrength(value.getCsq());
//        device.setSatellites(value.getS());
//        device.setQuality(value.getQ());
//        int i = asDeviceService.updateLocation(device);
//        return i;
//    }

    /** 坐标转换 */
    @NotNull
    private double[] coordinateConvert(LogEntry.LocationValue value) {
        BigDecimal lon = new BigDecimal(value.getLon());
        BigDecimal lat = new BigDecimal(value.getLat());
//        log.info("WGS84经纬度（未计算）：" + lon + "---" + lat);
        // 除以100
        lon = lon.divide(new BigDecimal(100), 10, RoundingMode.HALF_UP);
        lat = lat.divide(new BigDecimal(100), 10, RoundingMode.HALF_UP);
//        log.info("WGS84经纬度（除以100后）：" + lon + "---" + lat);
        // 取出lon中后面的小数点
        String[] lonStr = getDecimalPart(lon);
        String[] latStr = getDecimalPart(lat);
//        log.info("WGS84经纬度（截取小数点）：" + lonStr[0] + "---" + lonStr[1] + "---"+ latStr[0]+"---"+ latStr[1]);
        // 再将结果乘以5/3
        String lon2 = "0."+ lonStr[1];
        String lat2 = "0."+ latStr[1];
        BigDecimal lons = new BigDecimal(lon2).multiply(new BigDecimal(5).divide(new BigDecimal(3), 8, RoundingMode.HALF_UP));
        BigDecimal lats = new BigDecimal(lat2).multiply(new BigDecimal(5).divide(new BigDecimal(3), 8, RoundingMode.HALF_UP));
        BigDecimal lo = new BigDecimal(lonStr[0]).add(lons);
        BigDecimal la = new BigDecimal(latStr[0]).add(lats);
//        log.info("WGS84经纬度（计算后）：" + lo + "---" + la);
        lo = lo.setScale(8, RoundingMode.HALF_UP);
        la = la.setScale(8, RoundingMode.HALF_UP);
//        log.info("WGS84经纬度（保留8为小数）：" + lo + "---" + la);
        double[] doubles = GpsCoordinateUtils.calWGS84toGCJ02(la.doubleValue(), lo.doubleValue());
        return doubles;
    }

    /** 超出运营区断电*/
    private void outAreaOutage(LogEntry.LocationValue value, AsDevice device, BigDecimal lon, BigDecimal lat, EtOperatingArea area, String isAdminUnlocking, boolean noRidingArea) throws UnsupportedEncodingException, NoSuchAlgorithmException, InvalidKeyException {
        String nearBoundaryDistanceConfig = sysConfigService.selectConfigByKey("near.boundary.distance");// 靠近运营区边界时的播报距离
        log.info("靠近运营区边界时的播报距离==================：" + nearBoundaryDistanceConfig);
        double nearBoundaryDistance = Double.parseDouble(nearBoundaryDistanceConfig);
        String exceedArea = sysConfigService.selectConfigByKey("exceed.area.distance");// 超出运营区外断电距离
        log.info("超出运营区外断电距离================：" + exceedArea);
        int exceedDistance  = Integer.parseInt(exceedArea);

        // 创建多边形对象
        Geometry polygon = GeoUtils.fromWkt(area.getBoundary());
        // 是否在缩短后的运营区边界内
        boolean inPolygonWithTolerance = GeoUtils.isInPolygonWithShorten(lon.toString(), lat.toString(), polygon, nearBoundaryDistance);
        log.info("=========================是否在缩短后的运营区边界内：{}，车辆定位=【{},{}】边界=【{}】",inPolygonWithTolerance,lon,lat,polygon);
        if(!inPolygonWithTolerance && !isAdminUnlocking.equals("1") && device.getStatus().equals(ServiceConstants.VEHICLE_STATUS_IN_USING)){//没有在缩短后的运营区边界内
            boolean inPolygonWithTolerance1 = GeoUtils.isInPolygonWithTolerance(lon.toString(), lat.toString(), polygon, 0);// 是否在运营区内
            boolean isNearBoundary = GeoUtils.isInPolygonWithTolerance(lon.toString(), lat.toString(), polygon, exceedDistance );
            if(inPolygonWithTolerance1){//是否在运营区边界内
                // 在靠近运营区边界时发警报
                log.info("靠近运营区边界发警告命令===============--SN：" + device.getSn());
                sendCommandWithCooldown(device, IotConstants.COMMAND_PLAY2, "靠近运营区边界");
                returnPower(device, noRidingArea);
            }else if(isNearBoundary){ // 是否在超出运营区边界多少米内
                //在20米范围内，发报警
                log.info("超出运营区"+exceedDistance+"米内发送警告命令==============--SN：" + device.getSn());
                sendCommandWithCooldown(device, IotConstants.COMMAND_PLAY3, "超出运营区" + exceedDistance + "米内");
            }else{
                // 超出运营区外断电
                String areaOutOutage = area.getAreaOutOutage();
                if (areaOutOutage.equals("1")) { // 超出营运区断电
                    log.info("超出营运区断电命令=================--SN：" + device.getSn());
                    sendCommandWithCooldown(device, IotConstants.COMMAND_QLOSE + IotConstants.COMMAND_FREQUENCY_5, "超出营运区断电");
                    device.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                    int updateAsDevice = asDeviceService.updateAsDevice(device);
                    if (updateAsDevice > 0) {
                        log.info("超出营运区断电命令--更新设备锁状态成功SN：" + device.getSn());
                    }
                }
            }
        }else{
            log.info("在缩短后的运营区边界内====================：" + device.getSn());
            returnPower(device, noRidingArea);
        }
    }

    /** 带20秒的冷却时间 */
    private void sendCommandWithCooldown(AsDevice device, String command, String message) {
        String deviceKey = device.getSn() + "_" + command; // 使用设备SN和命令作为唯一标识
        long currentTime = System.currentTimeMillis();
        Long lastCommandTime = commandCooldownMap.get(deviceKey);

        if (lastCommandTime == null || (currentTime - lastCommandTime) > COOLDOWN_PERIOD_MS) {
            // 冷却时间已过，可以发送命令
            try {
                asDeviceService.sendCommand(device.getMac(), Token.getToken(), command, message, null, null);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            } catch (NoSuchAlgorithmException e) {
                e.printStackTrace();
            } catch (InvalidKeyException e) {
                e.printStackTrace();
            }
            commandCooldownMap.put(deviceKey, currentTime); // 更新命令发送时间
        } else {
            log.info("命令在冷却时间内，不执行命令==============--SN：" + device.getSn());
        }
    }

    /** 返回运营区上电 */
    private void returnPower(AsDevice device, boolean noRidingArea) throws UnsupportedEncodingException, NoSuchAlgorithmException, InvalidKeyException {
        // 判断该车辆是否在进行中的订单,并且车辆的锁状态是关，状态是骑行中
        Boolean inOrderBySn = etOrderService.isInOrderBySn(device.getSn());
        if (inOrderBySn && ServiceConstants.VEHICLE_STATUS_IN_USING.equals(device.getStatus()) && ServiceConstants.LOCK_STATUS_CLOSE.equals(device.getLockStatus()) && !noRidingArea) { // 有正在骑行的订单，给车辆上电
            log.info("返回营运区上电，有正在骑行的订单，给车辆上电--SN：" + device.getSn());
            asDeviceService.sendCommand(device.getMac(), Token.getToken(), IotConstants.COMMAND_OPEN+IotConstants.COMMAND_FREQUENCY_5, "返回营运区上电",null,null);
            // 更新车辆状态和锁状态
            /** 3.更新车辆状态*/
            device.setLockStatus(ServiceConstants.LOCK_STATUS_OPEN);
            device.setStatus(ServiceConstants.VEHICLE_STATUS_IN_USING);
            device.setIsAdminUnlocking("0");
            int i1 = asDeviceMapper.updateAsDevice(device);
            if(i1>1){
                log.info("【返回营运区上电】更新车辆状态成功");
            }
        }
    }

    /* 禁行区内断电*/
    private boolean isNoRidingArea(LogEntry.LocationValue value, AsDevice device, EtOperatingArea area, String isAdminUnlocking) throws UnsupportedEncodingException, NoSuchAlgorithmException, InvalidKeyException {
        boolean noRidingArea = asDeviceService.isNoRidingArea(device.getSn(), device.getAreaId());
        if(noRidingArea){
            String noRidingOutage = area.getNoRidingOutage();
            if (noRidingOutage.equals("1") && value.getStatus() != 3 && !isAdminUnlocking.equals("1")) { // 禁行区内断电
                log.info("禁行区内断电命令--SN：" + device.getSn());
                sendCommandWithCooldown(device, IotConstants.COMMAND_QLOSE + IotConstants.COMMAND_FREQUENCY_5, "禁行区内断电");
                device.setLockStatus(ServiceConstants.LOCK_STATUS_CLOSE);
                int updateAsDevice = asDeviceMapper.updateAsDevice(device);
                if (updateAsDevice > 0) {
                    log.info("禁行区内断电--更新设备锁状态成功SN：" + device.getSn());
                }
            }
        }else{
            //是否在禁行区20米范围内
            boolean inPolygon = asDeviceService.isNoRidingAreaWithTolerance(device.getSn(), device.getAreaId(),30);
            if (inPolygon && !isAdminUnlocking.equals("1")) {
                log.info("距离禁行区20米内发送警告命令--SN：" + device.getSn());
                sendCommandWithCooldown(device, IotConstants.COMMAND_PLAY2, "距离禁行区30米内");
            }
        }
        return noRidingArea;
    }

    private void lockSynchronization(String msg, AsDevice asDevice, LogEntry logEntry, LogEntry.LocationValue value, AsDevice device, BigDecimal lon, BigDecimal lat) {
        Integer status = value.getStatus();
        if(status == 1){//上电运行
            log.info("上电运行：" + logEntry.getDevName());
            if(BigDecimal.ZERO.compareTo(lon) != 0 && BigDecimal.ZERO.compareTo(lat) != 0){
                //获取当前正在骑行中的订单
                boolean currentOrderNum = etOrderService.isInOrderBySn(device.getSn());
                if(!currentOrderNum){
                    long currentTime = System.currentTimeMillis();
                    long lastTime = lastCommandTime.get();// 上一次命令时间，5分账内不再执行 ‘车辆锁同步关锁’ 命令
                    long difference = System.currentTimeMillis() - logEntry.getAt();
                    log.info("当前时间戳：【"+System.currentTimeMillis()+"】,消息时间：【"+ logEntry.getAt()+"】,时间差：【"+difference+"】");
                    if (difference < 60 * 1000 && ServiceConstants.LOCK_STATUS_CLOSE.equals(device.getLockStatus())) {// 消息时间和当前时间，时间差在1分钟内，且锁状态为关闭，则发送命令
                        if (currentTime - lastTime >= COMMAND_COOLDOWN_MS) {
                            try {
                                asDeviceService.sendCommand(asDevice.getMac(), Token.getToken(), IotConstants.COMMAND_CLOSE + IotConstants.COMMAND_FREQUENCY_3600, "车辆锁同步关锁", null, null, msg);
                            } catch (UnsupportedEncodingException e) {
                                e.printStackTrace();
                            } catch (NoSuchAlgorithmException e) {
                                e.printStackTrace();
                            } catch (InvalidKeyException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
            }
        }
    }


    private void replacementOrder(AsDevice device, EtOperatingArea area) {
        Integer replacementOrder = area.getAutoReplacementOrder();
        Boolean aBoolean = etAdminOrderService.checkOrderUnique(device.getSn());
        if (Integer.parseInt(device.getRemainingPower()) < replacementOrder && !aBoolean) {
            /** 生成换电工单 */
            EtAdminOrder adminOrder = new EtAdminOrder();
            adminOrder.setDeviceMac(device.getMac());
            adminOrder.setSn(device.getSn());
            adminOrder.setBeforeElectric(Integer.parseInt(device.getRemainingPower()));
            adminOrder.setOrderNo(IdUtils.getOrderNo("hd"));
            adminOrder.setAreaId(area.getAreaId());
            adminOrder.setStatus(ServiceConstants.REPLACEMENT_ELECTRICITY_STATUS_UNDER_WAY);
            adminOrder.setIsEffective(ServiceConstants.REPLACEMENT_ELECTRICITY_IS_EFFECTIVE_YES);
            SysUser sysUser = sysUserService.selectUserByUserName("wx");//获取用户名为“wx”的维修人员
            adminOrder.setAdminId(sysUser.getUserId());
            adminOrder.setAdminName(sysUser.getUserName());
            adminOrder.setLatitude(device.getLatitude());
            adminOrder.setLongitude(device.getLongitude());
            adminOrder.setType("2");
            int i1 = etAdminOrderService.insertEtAdminOrder(adminOrder);
            if (i1 > 0) {
                log.info("生成换电工单成功");
            }else{
                throw new ServiceException("生成换电工单失败");
            }
            /** 改变车辆状态 */
            if (asDeviceService.updateAsDevice(device) > 0) {
                log.info("车辆状态改成换电中");
            }else{
                throw new ServiceException("车辆状态更新失败");
            }
        }
    }

    /**
     * 异步更新在线状态
     */
    private void asynchronousUpdateOnlineStatus(AsDevice device) {
        //开异步线程：1.更新在线状态  2.记录在线、离线日志
        scheduledExecutorService.schedule(() -> {
            if(device.getOnlineStatus().equals(ServiceConstants.VEHICLE_STATUS_OFFLINE)){
                log.info("【接收onenet推送】异步更新在线状态"+JSON.toJSONString(device));
                device.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_ONLINE);
                int i = asDeviceService.updateAsDevice(device);
                if(i>0){
                    log.info("【接收onenet推送】异步保存在线状态成功");
                }
                EtOnlineLog etOnlineLog = new EtOnlineLog();
                etOnlineLog.setMac(device.getMac());
                etOnlineLog.setSn(device.getSn());
                etOnlineLog.setOnlineStatus(ServiceConstants.VEHICLE_STATUS_ONLINE);
                etOnlineLog.setLongitude(device.getLongitude());
                etOnlineLog.setLatitude(device.getLatitude());
                etOnlineLog.setCreateTime(DateUtils.getNowDate());
                int i1 = etOnlineLogService.insertEtOnlineLog(etOnlineLog);
                if(i1>0){
                    log.info("【接收onenet推送】异步保存在线日志成功");
                }
            }
        }, 0, TimeUnit.SECONDS);
    }

    private boolean isLastPointSame(JSONArray jsonArray, BigDecimal lon, BigDecimal lat) {
        if (jsonArray.size() > 0) {
            JSONArray lastPoint = (JSONArray) jsonArray.get(jsonArray.size() - 1);
            return lastPoint.get(0).equals(lon) && lastPoint.get(1).equals(lat);
        }
        return false;
    }

//    public void optimizeRoute(JSONArray jsonArray, JSONArray newPoint, BigDecimal lon, BigDecimal lat, EtOrder etOrder) {
//        if (isLastPointSame(jsonArray, lon, lat)) {
//            log.info("获取到的定位与最后一个定位相同，不添加线路");
//        } else {
//            jsonArray.add(newPoint);
//            String tripRouteStr = jsonArray.toJSONString();
////            log.info("更新行程jsonArray：" + tripRouteStr);
//            etOrder.setTripRouteStr(tripRouteStr);
//
//            Geometry geometry = GeoUtils.toGeometryByLinearRing(tripRouteStr);
//            String wkt = GeoUtils.wkt(geometry);
//            etOrder.setTripRoute(wkt);
//
//            int updateEtOrderResult = etOrderService.updateEtOrder(etOrder);
//            if (updateEtOrderResult > 0) {
//                log.info("更新行程线路成功==========================>" + etOrder.getSn());
//            } else {
//                log.info("更新行程线路失败==========================>" + etOrder.getSn());
//            }
//        }
//    }


    /**
     * 功能说明： URL&Token验证接口。如果验证成功返回msg的值，否则返回其他值。
     * @param msg 验证消息
     * @param nonce 随机串
     * @param signature 签名
     * @return msg值
     */

    @RequestMapping(value = "/receive", method = RequestMethod.GET)
    @ResponseBody
    public String check(@RequestParam(value = "msg") String msg,
                        @RequestParam(value = "nonce") String nonce,
                        @RequestParam(value = "signature") String signature) throws UnsupportedEncodingException {

        log.info("check方法接收到参数：: msg:{} nonce{} signature:{}",msg,nonce,signature);
        if (Util.checkToken(msg,nonce,signature,token)){
            log.info("校验成功",msg,nonce,signature);
            return msg;
        }else {
            log.info("校验失败",msg,nonce,signature);
            return "error";
        }

    }

    private static String[] getDecimalPart(BigDecimal number) {
        // 将BigDecimal转换为字符串
        String numberStr = number.toPlainString();

        // 找到小数点的位置
        int indexOfDecimal = numberStr.indexOf(".");

        // 初始化结果数组
        String[] parts = new String[2];

        // 如果有小数点
        if (indexOfDecimal >= 0) {
            parts[0] = numberStr.substring(0, indexOfDecimal); // 整数部分
            parts[1] = numberStr.substring(indexOfDecimal + 1); // 小数部分
        } else {
            // 如果没有小数点，整数部分为整个字符串，小数部分为空
            parts[0] = numberStr;
            parts[1] = "";
        }

        return parts;
    }
}
