package com.ruoyi.web.controller.system;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.vo.RechargeVo;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.IEtOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.List;

/**
 * 订单Controller
 *
 * @author 邱贞招
 * @date 2024-04-08
 */
@RestController
@RequestMapping("/system/order")
public class EtOrderController extends BaseController
{
    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private TokenService tokenService;

    /**
     * 查询订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:list')")
    @GetMapping("/list")
    public TableDataInfo list(EtOrder etOrder)
    {
        startPage();
        List<EtOrder> list = etOrderService.selectEtOrderListWithIsolate(etOrder);
        return getDataTable(list);
    }

    /**
     * 查询充值/退款订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:list')")
    @GetMapping("/rechargeList")
    public TableDataInfo rechargeList(EtOrder etOrder)
    {
        startPage();
        List<RechargeVo> list = etOrderService.rechargeList(etOrder);
        return getDataTable(list);
    }

    /**
     * 导出订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:export')")
    @Log(title = "订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EtOrder etOrder)
    {
        List<EtOrder> list = etOrderService.selectEtOrderListWithIsolate(etOrder);
        ExcelUtil<EtOrder> util = new ExcelUtil<EtOrder>(EtOrder.class);
        util.exportExcel(response, list, "订单数据");
    }

    /**
     * 押金退款
     */
    @Transactional
    @Log(title = "押金退款", businessType = BusinessType.REFUND)
    @PostMapping("/depositRefund/{orderNo}")
    public AjaxResult depositRefund(@PathVariable("orderNo") String orderNo)
    {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(ObjectUtil.isNotNull(order)){
            Long userId = order.getUserId();
            if(userId!=null){
                AsUser asUser = asUserService.selectUserById(userId);
                if(asUser!=null){
                    // 更新用户并更新缓存
                    asUser.setBalance(BigDecimal.ZERO);
                    if (asUserService.updateUserProfile(asUser) > 0)
                    {
                        logger.info("【提现金额】更新用户缓存:"+ JSON.toJSON(asUser));
                    }
                }
            }
            logger.info("押金退款：【{}】", order.getPayFee());
            int withdraw = etOrderService.withdraw(order, order.getPayFee());
            return toAjax(withdraw);
        }else{
            throw new ServiceException("订单不存在");
        }
    }

    /**
     * 还车审核通过
     *  改状态，退押金
     */
    @Transactional
    @Log(title = "还车审核通过", businessType = BusinessType.PASSAUDIT)
    @PostMapping("/passAudit/{orderNo}")
    public AjaxResult passAudit(@PathVariable("orderNo") String orderNo)
    {
        logger.info("还车审核通过请求：【{}】", orderNo);
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        if(ObjectUtil.isNotNull(order)){
            order.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
            return toAjax(etOrderService.passAudit(order));
        }else{
            throw new ServiceException("订单不存在");
        }
    }

    /**
     * 获取订单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:order:query')")
    @GetMapping(value = "/{orderId}")
    public AjaxResult getInfo(@PathVariable("orderId") Long orderId)
    {
        return success(etOrderService.selectEtOrderByOrderId(orderId));
    }

    /**
     * 新增订单
     */
    @PreAuthorize("@ss.hasPermi('system:order:add')")
    @Log(title = "订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EtOrder etOrder)
    {
        return toAjax(etOrderService.insertEtOrder(etOrder));
    }

    /**
     * 修改订单
     */
    @PreAuthorize("@ss.hasPermi('system:order:edit')")
    @Log(title = "订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EtOrder etOrder)
    {
        return toAjax(etOrderService.updateEtOrder(etOrder));
    }

    /**
     * 删除订单
     */
    @PreAuthorize("@ss.hasPermi('system:order:remove')")
    @Log(title = "订单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{orderIds}")
    public AjaxResult remove(@PathVariable Long[] orderIds)
    {
        return toAjax(etOrderService.deleteEtOrderByOrderIds(orderIds));
    }
}
