package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.EtCoupon;
import com.ruoyi.system.domain.EtCouponUserLog;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.vo.CouponVo;
import com.ruoyi.system.domain.vo.PrepayWithRequestPaymentResponseVo;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.mapper.EtCouponClaimLogMapper;
import com.ruoyi.system.mapper.EtCouponMapper;
import com.ruoyi.system.mapper.EtOrderMapper;
import com.ruoyi.system.service.IEtCouponService;
import com.ruoyi.system.service.IWxPayService;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 优惠券Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-08-05
 */
@Service
@Slf4j
public class EtCouponServiceImpl implements IEtCouponService
{
    @Resource
    private EtCouponMapper etCouponMapper;

    @Resource
    private AsUserMapper userMapper;

    @Resource
    private EtOrderMapper etOrderMapper;

    @Autowired
    private IWxPayService wxPayService;

    @Resource
    private EtCouponClaimLogMapper etCouponClaimLogMapper;

    /**
     * 查询优惠券
     *
     * @param couponId 优惠券主键
     * @return 优惠券
     */
    @Override
    public EtCoupon selectEtCouponByCouponId(Long couponId)
    {
        return etCouponMapper.selectEtCouponByCouponId(couponId);
    }

    /**
     * 查询优惠券列表
     *
     * @param etCoupon 优惠券
     * @return 优惠券
     */
    @Override
    public List<EtCoupon> selectEtCouponList(EtCoupon etCoupon)
    {
        return etCouponMapper.selectEtCouponList(etCoupon);
    }

    /**
     * 新增优惠券
     *
     * @param etCoupon 优惠券
     * @return 结果
     */
    @Override
    public int insertEtCoupon(EtCoupon etCoupon)
    {
        etCoupon.setCreateTime(DateUtils.getNowDate());
        return etCouponMapper.insertEtCoupon(etCoupon);
    }

    /**
     * 修改优惠券
     *
     * @param etCoupon 优惠券
     * @return 结果
     */
    @Override
    public int updateEtCoupon(EtCoupon etCoupon)
    {
        return etCouponMapper.updateEtCoupon(etCoupon);
    }

    /**
     * 批量删除优惠券
     *
     * @param couponIds 需要删除的优惠券主键
     * @return 结果
     */
    @Override
    public int deleteEtCouponByCouponIds(Long[] couponIds)
    {
        return etCouponMapper.deleteEtCouponByCouponIds(couponIds);
    }

    /**
     * 删除优惠券信息
     *
     * @param couponId 优惠券主键
     * @return 结果
     */
    @Override
    public int deleteEtCouponByCouponId(Long couponId)
    {
        return etCouponMapper.deleteEtCouponByCouponId(couponId);
    }

    /**
     * 优惠券购买
     * @param userId
     * @param couponId
     * @return
     */
    @Override
    @Transactional
    public PrepayWithRequestPaymentResponse couponBuy(Long userId, Long couponId) {
        EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(couponId);
        if(ObjectUtils.isEmpty(etCoupon)){
            throw new ServiceException("优惠券【"+couponId+"】不存在");
        }
        AsUser asUser = userMapper.selectUserById(userId);
        if(ObjectUtil.isNull(asUser)){
            throw new ServiceException("用户【"+userId+"】不存在");
        }
        // 2. 增加订单记录，类型是优惠券订单
        EtOrder order = createOrder(etCoupon, asUser);
        PrepayWithRequestPaymentResponse response = wxPayService.prepayWithRequestPayment(order);
        PrepayWithRequestPaymentResponseVo responseVo = new PrepayWithRequestPaymentResponseVo();
        BeanUtils.copyProperties(response,responseVo);
        responseVo.setSessionId(IdUtils.randomUUIDByDigit(8));
        responseVo.setOrderNo(order.getOrderNo());
        return responseVo;
    }

    /**
     * 下发优惠券
     *
     * @param couponVo 优惠券
     * @return 结果
     */
    @Override
    public int issue(CouponVo couponVo) {
        Long[] userIds = couponVo.getUserIds();
        Long couponId = Long.parseLong(couponVo.getCouponId());
        for (Long userId : userIds) {
            AsUser asUser = userMapper.selectUserById(userId);
            if(ObjectUtil.isNull(asUser)){
                throw new ServiceException("用户【"+userId+"】不存在");
            }
            EtCoupon etCoupon = etCouponMapper.selectEtCouponByCouponId(couponId);
            if(ObjectUtil.isNull(etCoupon)){
                throw new ServiceException("优惠券【"+couponId+"】不存在");
            }
            if(asUser.getVipType().equals(ServiceConstants.COUPON_TYPE_VIP_CARD) && etCoupon.getType().equals(ServiceConstants.COUPON_TYPE_TIME_CARD)){
                throw new ServiceException("用户【"+asUser.getUserName()+"】已经是贵宾卡，无需添加会员卡");
            }
            EtCouponUserLog etCouponUserLog = new EtCouponUserLog();
            etCouponUserLog.setCouponId(couponId);
            etCouponUserLog.setUserId(userId);
            etCouponUserLog.setUserName(asUser.getUserName());
            etCouponUserLog.setCreateTime(DateUtils.getNowDate());
            etCouponUserLog.setGainMethod(ServiceConstants.COUPON_GAIN_METHOD_ISSUE);
            etCouponUserLog.setAreaId(etCoupon.getAreaId());
            etCouponUserLog.setAreaName(etCoupon.getAreaName());
            etCouponUserLog.setLimitNum(Integer.parseInt(etCoupon.getLimitNum()));
            if(ObjectUtil.isNull(etCoupon.getValidityUnit())){
                throw new ServiceException("优惠券【有效期单位--validityUnit】为空");
            }
            if(ObjectUtil.isNull(etCoupon.getValidityValue())){
                throw new ServiceException("优惠券【有效期值--validityValue】为空");
            }
            etCouponUserLog.setExpirationTime(DateUtils.addValidity(new Date(),etCoupon.getValidityUnit(),etCoupon.getValidityValue()));
            if(ServiceConstants.COUPON_TYPE_TIME_CARD.equals(etCoupon.getType()) || ServiceConstants.COUPON_TYPE_VIP_CARD.equals(etCoupon.getType())){
                AsUser asUser1 = new AsUser(userId);
                asUser1.setVipType(etCoupon.getType());
                Date expirationTime;
                if(ObjectUtil.isNull(asUser.getExpirationTime())){
                    expirationTime = DateUtils.getNowDate();
                }else{
                    expirationTime = asUser.getExpirationTime();
                }
                asUser1.setExpirationTime(DateUtils.addValidity(expirationTime,etCoupon.getValidityUnit(),etCoupon.getValidityValue()));
                asUser1.setUserId(userId);
                int updateUser = userMapper.updateUser(asUser1);
            }
            int i1 = etCouponClaimLogMapper.insertEtCouponClaimLog(etCouponUserLog);
            if(i1 == 0){
                throw new ServiceException("保存优惠券领取记录失败");
            }
        }
        return 1;
    }


    private EtOrder createOrder(EtCoupon etCoupon,AsUser user) {
        EtOrder etOrder = new EtOrder();
        etOrder.setOrderNo(IdUtils.getOrderNo("yhj"));
        etOrder.setUserId(user.getUserId());
        etOrder.setUserName(user.getUserName());
        etOrder.setMark("优惠券购买");
        etOrder.setCreateTime(DateUtils.getNowDate());
        etOrder.setAreaId(etCoupon.getAreaId());
        /** 订单类型 1骑行 2预约 3套餐 4押金 根据类型不同下单的参数也不同，不同的参数有： 金额、类型、rule_id（套餐id）*/
        etOrder.setArea(etCoupon.getAreaName());
        etOrder.setType(ServiceConstants.ORDER_TYPE_COUPON);
        etOrder.setTotalFee(etCoupon.getRetailPrice());
        etOrder.setPayFee(etCoupon.getRetailPrice());
        etOrder.setCouponId(etCoupon.getCouponId());
        int i = etOrderMapper.insertEtOrder(etOrder);
        if(i == 0){
            throw new ServiceException("订单生成失败");
        }
        return etOrder;
    }
}
